# -*- coding: utf-8 -*-
import datetime
from django.db import models

from cms.models.fields import PageField
from parler.models import TranslatedField
from filer.fields.image import FilerImageField
from djangocms_text_ckeditor.fields import HTMLField
from cms.models.fields import PlaceholderField

from aldryn_common.admin_fields.sortedm2m import SortedM2MModelField
from allink_core.core.models import (
    AllinkCategoryFieldsModel,
    AllinkBaseTranslatableModel,
    AllinkBaseTranslatedFieldsModel,
    AllinkBaseAppContentPlugin,
    AllinkSimpleRegistrationFieldsModel,
    AllinkTimeFramedModel,
)
from allink_core.core.loading import get_class

AllinkEventsManager = get_class('events.managers', 'AllinkEventsManager')


class BaseEvents(AllinkTimeFramedModel, AllinkCategoryFieldsModel, AllinkBaseTranslatableModel):
    slug_source_field_name = 'title'

    title = TranslatedField(any_language=True)
    lead = TranslatedField()

    preview_image = FilerImageField(
        verbose_name='Preview Image',
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        related_name='%(app_label)s_%(class)s_preview_image',
    )
    template = models.CharField(
        'Template',
        help_text='Choose a template.',
        max_length=50,
        blank=True,
        null=True,
    )
    header_placeholder = PlaceholderField(
        'events_header',
        related_name='%(app_label)s_%(class)s_header_placeholder'
    )
    content_placeholder = PlaceholderField(
        'events_content',
        related_name='%(app_label)s_%(class)s_content_placeholder'
    )

    costs = TranslatedField()

    form_enabled = models.BooleanField(
        'Event Form enabled',
        default=True
    )
    entry_date = models.DateTimeField(
        'Entry Date',
    )
    location = models.ForeignKey(
        'locations.Locations',
        on_delete=models.PROTECT,
        blank=True,
        null=True,
        related_name='events'
    )

    objects = AllinkEventsManager()

    class Meta:
        abstract = True
        app_label = 'events'
        verbose_name = 'Event'
        verbose_name_plural = 'Events'

    def __str__(self):
        return '%s %s' % (self.title, self.entry_date.strftime('%d.%m.%Y %H:%M:%S'))

    def show_registration_form(self):
        if getattr(self, 'entry_date'):
            if self.entry_date < datetime.now().date():
                return False
        if self.form_enabled:
            return True
        else:
            return False


class BaseEventsTranslation(AllinkBaseTranslatedFieldsModel):
    master = models.ForeignKey(
        'events.Events',
        on_delete=models.CASCADE,
        related_name='translations',
        null=True
    )

    title = models.CharField(
        max_length=255
    )
    lead = HTMLField(
        'Lead Text',
        help_text='Teaser text that in some cases is used in the list view and/or in the detail view.',
        blank=True,
        null=True,
    )
    costs = models.CharField(
        max_length=255,
        help_text='Costs',
        blank=True,
        null=True,
    )

    class Meta:
        abstract = True
        app_label = 'events'


class BaseEventsAppContentPlugin(AllinkBaseAppContentPlugin):
    # FILTERING
    DEFAULT = 'default'
    UPCOMING = 'upcoming'
    PAST = 'past'

    FILTERING = (
        (DEFAULT, '---------'),
        (UPCOMING, 'upcoming'),
        (PAST, 'past'),
    )

    manual_entries = SortedM2MModelField(
        'events.Events',
        blank=True,
        help_text=('Select and arrange specific entries, or, leave blank to select all. (If '
                   'manual entries are selected the category filtering will be applied as well.)')
    )
    apphook_page = PageField(
        verbose_name='Apphook Page',
        null=True,
        blank=True,
        on_delete=models.PROTECT,
        help_text='If provided, this Apphook-Page will be used to generate the detail link.',
    )

    class Meta:
        abstract = True
        app_label = 'events'

    def _apply_filtering_to_queryset_for_display(self, queryset):
        # upcoming
        if self.manual_filtering == BaseEventsAppContentPlugin.UPCOMING:
            return queryset.upcoming_entries()
        # past
        elif self.manual_filtering == BaseEventsAppContentPlugin.PAST:
            return queryset.past_entries()
        return queryset

    def save(self, *args, **kwargs):
        super(BaseEventsAppContentPlugin, self).save(*args, **kwargs)


class BaseEventsRegistration(AllinkSimpleRegistrationFieldsModel):
    event = models.ForeignKey(
        'events.Events',
        on_delete=models.CASCADE,
        null=True
    )

    # terms = models.ForeignKey(
    #     'allink_terms.AllinkTerms',
    #     verbose_name=('I have read and accept the terms and conditions.'),
    #     blank=True,
    #     null=True
    # )

    class Meta:
        abstract = True
        app_label = 'events'

    def __str__(self):
        return '%s %s' % (self.first_name, self.last_name)



# -*- coding: utf-8 -*-
from django.contrib import admin
from django import forms
from adminsortable.admin import SortableAdmin
from allink_core.core.loading import get_model
from parler.admin import TranslatableAdmin
from allink_core.core.admin import AllinkMediaAdminMixin, AllinkSEOAdminMixin, AllinkCategoryAdminMixin, \
    AllinkTeaserAdminMixin


Testimonials = get_model('testimonials', 'Testimonials')


@admin.register(Testimonials)
class TestimonialsAdmin(AllinkMediaAdminMixin, AllinkSEOAdminMixin, AllinkCategoryAdminMixin, AllinkTeaserAdminMixin,
                        TranslatableAdmin, SortableAdmin):
    search_fields = ('first_name', 'last_name',)
    list_display = ('first_name', 'last_name', 'status', 'all_categories_column', 'created', 'modified')

    def get_fieldsets(self, request, obj=None):
        fieldsets = (
            (None, {
                'fields': (
                    'status',
                    ('first_name', 'last_name'),
                    'lead',
                    'slug',
                    'created',
                    'preview_image',
                    'job_function',
                    'company_name',
                ),
            }),
        )
        fieldsets += self.get_category_fieldsets()
        fieldsets += self.get_teaser_fieldsets()
        fieldsets += self.get_seo_fieldsets()
        return fieldsets

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == 'lead':
            kwargs['widget'] = forms.Textarea
        return super(TestimonialsAdmin, self).formfield_for_dbfield(db_field, **kwargs)



# -*- coding: utf-8 -*-
from allink_core.apps.testimonials.abstract_models import BaseTestimonials, BaseTestimonialsTranslation, \
    BaseTestimonialsAppContentPlugin
from allink_core.core.loading import get_model
from allink_core.core.loading import is_model_registered

__all__ = []


if not is_model_registered('testimonials', 'Testimonials'):
    class Testimonials(BaseTestimonials):
        pass

    __all__.append('Testimonials')


if not is_model_registered('testimonials', 'TestimonialsTranslation'):
    class TestimonialsTranslation(BaseTestimonialsTranslation):
        pass

    __all__.append('TestimonialsTranslation')


if not is_model_registered('testimonials', 'TestimonialsAppContentPlugin'):
    class TestimonialsAppContentPlugin(BaseTestimonialsAppContentPlugin):
        data_model = get_model('testimonials', 'Testimonials')

    __all__.append('TestimonialsAppContentPlugin')



# -*- coding: utf-8 -*-
from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool


class ContactApphook(CMSApp):
    name = "Contact Apphook"
    app_name = 'contact'

    def get_urls(self, page=None, language=None, **kwargs):
        return ['allink_core.apps.contact.urls']


apphook_pool.register(ContactApphook)



# -*- coding: utf-8 -*-
import requests
import json
import hashlib
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from raven import Client
from allink_core.core_apps.allink_mailchimp.config import MailChimpConfig


config = MailChimpConfig()


def check_response_status(response):
    client = Client(settings.RAVEN_CONFIG.get('dsn'))
    try:
        response.raise_for_status()
        return response.json()
    except requests.exceptions.HTTPError as err:
        client.captureException()
        raise requests.exceptions.HTTPError('Error: {} {}'.format(str(response.status_code), err))
    except ValueError as err:
        client.captureException()
        raise ValueError('Cannot decode json, got {}'.format(response.text), err)


def get_hash_md5(email):
    return hashlib.md5(email.encode('utf-8')).hexdigest()


def get_status_if_new():
    """
    You should pass the value “subscribed” in the API field instead of “pending”. Using “pending” will
    always send a double opt-in confirmation until that users confirms their subscription.
    By passing “subscribed” you will bypass this method.
    """
    return 'pending' if config.double_optin else 'subscribed'


def list_members_post(data, list_id=config.default_list_id, member_hash_email=None):
    member_hash = get_hash_md5(member_hash_email) if member_hash_email else get_hash_md5(data['email_address'])
    data = json.dumps(data)
    # POST new member
    try:
        response = requests.post(
            config.api_root + 'lists/{}/members/{}'.format(list_id, member_hash),
            auth=('apikey', config.apikey),
            data=data
        )
        check_response_status(response)
    except:
        pass


def list_members_put(data, list_id=config.default_list_id, member_hash_email=None):
    member_hash = get_hash_md5(member_hash_email) if member_hash_email else get_hash_md5(data['email_address'])
    data = json.dumps(data)
    try:
        # PUT new or modify existing member
        response = requests.put(
            config.api_root + 'lists/{}/members/{}'.format(list_id, member_hash),
            auth=('apikey', config.apikey),
            data=data
        )
        check_response_status(response)
    except:
        pass


def list_members_patch(data, list_id=config.default_list_id, member_hash_email=None):
    member_hash = get_hash_md5(member_hash_email) if member_hash_email else get_hash_md5(data['email_address'])
    data = json.dumps(data)
    # PATCH existing member
    response = requests.patch(
        config.api_root + 'lists/{}/members/{}'.format(list_id, member_hash),
        auth=('apikey', config.apikey),
        data=data
    )
    check_response_status(response)


def list_members_delete(data, list_id=config.default_list_id, member_hash_email=None):
    member_hash = get_hash_md5(member_hash_email) if member_hash_email else get_hash_md5(data['email_address'])
    # DELETE existing member
    response = requests.delete(
        config.api_root + 'lists/{}/members/{}'.format(list_id, member_hash),
        auth=('apikey', config.apikey),
    )
    check_response_status(response)



# -*- coding: utf-8 -*-
from django.template.loader import render_to_string

from allink_core.core.utils import get_display
from allink_core.core_apps.allink_mandrill.config import MandrillConfig
from allink_core.core_apps.allink_mandrill.helpers import send_transactional_email


def send_request_email(form, plugin):
    subject = render_to_string('contact/email/request_subject_internal.txt')
    template_content = [{}]

    config = MandrillConfig()

    r = '<br />'
    data = []
    for field in form.fields:
        if field != 'terms' and field != 'terms_accepted':
            if form.data.get(field):
                if hasattr(form.fields.get(field), 'choices'):
                    data.append((form.fields.get(field).label, get_display(form.data.get(field),
                                                                           form.fields.get(field).choices)))
                else:
                    data.append((form.fields.get(field).label, form.data.get(field)))

    subscriber = render_to_string('contact/email/request_subscriber.txt', {'data': data})
    subscriber = subscriber.replace('\r\n', r).replace('\n\r', r).replace('\r', r).replace('\n', r)
    # subscriber = subscriber.encode('utf-8')

    message = {
        'auto_html': None,
        'auto_text': None,
        'from_email': plugin.from_email_address,
        'from_name': config.get_default_from_name(),
        'global_merge_vars': [
            {'name': 'detail_link', 'content': 'hello'},
            {'name': 'subscriber', 'content': subscriber}
        ],
        'headers': {'Reply-To': plugin.from_email_address},
        'inline_css': True,
        'merge': True,
        'merge_language': 'mailchimp',
        'metadata': {'website': config.get_site_domain()},
        'preserve_recipients': True,
        'return_path_domain': None,
        'subject': subject,
        'to': [{
            'email': email_address,
            'name': config.get_default_from_name(),
            'type': 'to'
        } for email_address in plugin.internal_email_addresses if email_address],
        'track_clicks': True,
        'track_opens': True
    }
    send_transactional_email(message=message, template_name='hrcampus_request_internal_de',
                             template_content=template_content)


def send_request_confirmation_email(form, plugin):
    subject = render_to_string('contact/email/request_subject.txt')
    template_content = [{}]

    config = MandrillConfig()

    r = '<br />'
    data = []
    for field in form.fields:
        if field != 'terms' and field != 'terms_accepted':
            if form.data.get(field):
                if hasattr(form.fields.get(field), 'choices'):
                    data.append((form.fields.get(field).label,
                                 get_display(form.data.get(field), form.fields.get(field).choices)))
                else:
                    data.append((form.fields.get(field).label, form.data.get(field)))

    subscriber = render_to_string('contact/email/request_subscriber.txt', {'data': data})
    subscriber = subscriber.replace('\r\n', r).replace('\n\r', r).replace('\r', r).replace('\n', r)

    message = str(form.fields.get('message')).replace('\r\n', r).replace('\n\r', r).replace('\r', r).replace('\n', r)

    message = {
        'auto_html': None,
        'auto_text': None,
        'from_email': plugin.from_email_address,
        'from_name': config.get_default_from_name(),
        'global_merge_vars': [
            {'name': 'first_name', 'content': form.data.get('first_name')},
            {'name': 'last_name', 'content': form.data.get('last_name')},
            {'name': 'subscriber', 'content': subscriber},
            {'name': 'message', 'content': message}
        ],
        'google_analytics_campaign': 'Contact Request',
        'google_analytics_domains': [config.get_google_analytics_domains()],
        'headers': {'Reply-To': plugin.from_email_address},
        'inline_css': True,
        'merge': True,
        'merge_language': 'mailchimp',
        'metadata': {'website': config.get_site_domain()},
        'preserve_recipients': True,
        'return_path_domain': None,
        'subject': subject,
        'to': [{
            'email': form.data.get('email'),
            'name': '{} {}'.format(form.data.get('first_name'), form.data.get('last_name')),
            'type': 'to'
        }],
        'track_clicks': True,
        'track_opens': True
    }
    send_transactional_email(message=message, template_name='hrcampus_request_confirmation_de', translated=False,
                             template_content=template_content)



# -*- coding: utf-8 -*-
import logging

from django.core.management.base import BaseCommand, CommandError
from django.utils import six

from allink_core.core import customisation


class Command(BaseCommand):
    help = (
        "Create a local version of one of allink's app so it can "
        "be customised")

    def add_arguments(self, parser):
        parser.add_argument('app_label', help='The application to fork')
        parser.add_argument('target_path', help='The path to copy the files to')

    def handle(self, *args, **options):
        # Use a stdout logger
        logger = logging.getLogger(__name__)
        stream = logging.StreamHandler(self.stdout)
        logger.addHandler(stream)
        logger.setLevel(logging.DEBUG)

        app_label, folder_path = options['app_label'], options['target_path']
        try:
            customisation.fork_app(app_label, folder_path, logger)
        except Exception as e:
            raise CommandError(six.text_type(e))



# -*- coding: utf-8 -*-

from django.db import models
from cms.models.pluginmodel import CMSPlugin


class AllinkSignupFormPlugin(CMSPlugin):

    SIGNUP_FORMS = (
        ('simple', 'Simple'),
        ('advanced', 'Advanced')
    )

    signup_form = models.CharField(
        'Template',
        help_text='Choose a form.',
        max_length=50,
        choices=SIGNUP_FORMS,
        default=SIGNUP_FORMS[0]
    )

    def __str__(self):
        return ''



# -*- coding: utf-8 -*-
from django.views.generic import CreateView
from allink_core.core.loading import get_model, get_class
from allink_core.core.views import AllinkBasePluginLoadMoreView, AllinkBaseDetailView, AllinkBaseCreateView
from allink_core.core_apps.allink_mandrill.config import MandrillConfig


Events = get_model('events', 'Events')
EventsAppContentPlugin = get_model('events', 'EventsAppContentPlugin')
EventsRegistration = get_model('events', 'EventsRegistration')
EventsRegistrationForm = get_class('events.forms', 'EventsRegistrationForm')
send_registration_confirmation_email = get_class('events.email', 'send_registration_confirmation_email')
send_registration_email = get_class('events.email', 'send_registration_email')


config = MandrillConfig()


class EventsPluginLoadMore(AllinkBasePluginLoadMoreView):
    model = Events
    plugin_model = EventsAppContentPlugin


class EventsDetail(AllinkBaseDetailView):
    model = Events


class EventsRegistrationView(AllinkBaseCreateView):
    model = EventsRegistration
    form_class = EventsRegistrationForm
    template_name = 'events/forms/registration.html'

    def get_context_data(self, **kwargs):
        context = CreateView.get_context_data(self)
        context.update({
            'slug': self.kwargs.get('slug', None),
            'event_title': self.item.title
        })
        return context

    def get_initial(self):
        self.item = Events.objects.get(translations__slug=self.kwargs.get('slug', None))
        initial = super(EventsRegistrationView, self).get_initial()
        initial = initial.copy()
        initial['event'] = self.item
        # initial['terms'] = AllinkTerms.objects.get_published()  # for reference
        return initial

    def form_valid(self, form):
        response = super(EventsRegistrationView, self).form_valid(form)
        self.send_mail()
        return response

    def send_mail(self):
        send_registration_email(self.get_form(), self.item)
        send_registration_confirmation_email(self.get_form(), self.item)



# -*- coding: utf-8 -*-
from django import forms

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from allink_core.core_apps.allink_pdf.models import AllinkPdfPageBreakPlugin


class AllinkPdfPageBreakPluginForm(forms.ModelForm):

    class Meta:
        model = AllinkPdfPageBreakPlugin
        exclude = ('page', 'position', 'placeholder', 'language', 'plugin_type')


@plugin_pool.register_plugin
class CMSAllinkPageBreakPlugin(CMSPluginBase):
    model = AllinkPdfPageBreakPlugin
    name = 'PDF Page Break'
    module = 'Generic'
    form = AllinkPdfPageBreakPluginForm

    def get_render_template(self, context, instance, placeholder):
        template = 'allink_pdf/content.html'
        return template



# -*- coding: utf-8 -*-

# import django

# from django.core.management import call_command
# from django.utils.encoding import force_text
# from django.utils.six import StringIO

# from cms.api import add_plugin, create_page
# from cms.plugin_rendering import PluginContext

# from djangocms_helper.base_test import BaseTestCase
#
#
# class CMSAllinkContentPluginTest(BaseTestCase):
#     def test_link(self):
#         page = create_page(
#             title='help',
#             template='page.html',
#             language='en',
#         )
#
#         plugin = add_plugin(
#             page.placeholders.get(slot='content'),
#             'CMSAllinkContentPlugin',
#             'en',
#             external_link='http://example.com'
#         )
#         self.assertEqual(plugin.get_link(), 'http://example.com')

#     plugin = add_plugin(
#         page.placeholders.get(slot='content'),
#         'LinkPlugin',
#         'en',
#         external_link='http://example.com',
#         anchor='some-h1',
#     )
#     self.assertEqual(plugin.get_link(), 'http://example.com#some-h1')
#
#     plugin = add_plugin(
#         page.placeholders.get(slot='content'),
#         'LinkPlugin',
#         'en',
#         phone='555-123-555',
#     )
#     self.assertEqual(plugin.get_link(), 'tel:555-123-555')
#
#     plugin = add_plugin(
#         page.placeholders.get(slot='content'),
#         'LinkPlugin',
#         'en',
#         mailto='hello@example.com',
#     )
#     self.assertEqual(plugin.get_link(), 'mailto:hello@example.com')
#
#     plugin = add_plugin(
#         page.placeholders.get(slot='content'),
#         'LinkPlugin',
#         'en',
#         external_link='http://example.com',
#     )
#     self.assertEqual(plugin.get_link(), 'http://example.com')
#
#     plugin = add_plugin(
#         page.placeholders.get(slot='content'),
#         'LinkPlugin',
#         'en',
#         internal_link=page,
#     )
#     self.assertEqual(plugin.get_link(), '/en/')
#
#     plugin = add_plugin(
#         page.placeholders.get(slot='content'),
#         'LinkPlugin',
#         'en',
#         internal_link=page,
#         anchor='some-h1',
#     )
#     self.assertEqual(plugin.get_link(), '/en/#some-h1')
#
#     plugin = add_plugin(
#         page.placeholders.get(slot='content'),
#         'LinkPlugin',
#         'en',
#         name='some text',
#     )
#     self.assertEqual(plugin.get_link(), '')
#     self.assertEqual(force_text(plugin), 'some text')
#
# def test_makemigrations(self):
#     """
#     Fail if there are schema changes with no migrations.
#     """
#     app_name = 'djangocms_link'
#     out = StringIO()
#     call_command('makemigrations', dry_run=True, noinput=True, stdout=out)
#     output = out.getvalue()
#     self.assertNotIn(app_name, output, (
#         '`makemigrations` thinks there are schema changes without'
#         ' migrations.'
#     ))



# # -*- coding: utf-8 -*-
from django.urls import path
from allink_core.core.loading import get_class

EventsPluginLoadMore = get_class('events.views', 'EventsPluginLoadMore')
EventsDetail = get_class('events.views', 'EventsDetail')
EventsRegistrationView = get_class('events.views', 'EventsRegistrationView')


urlpatterns = [
    path('<int:page>/', EventsPluginLoadMore.as_view(), name='more'),
    path('<slug:slug>/', EventsDetail.as_view(), name='detail'),
    path('<slug:slug>/register/', EventsRegistrationView.as_view(), name='register'),
]



# # -*- coding: utf-8 -*-
# import os
# from django.test import TestCase
# from django.test.client import RequestFactory
# from django.template.loader import render
#
# from cms.api import add_plugin
# from cms.models import Placeholder
# from cms.plugin_rendering import ContentRenderer
#
# import allink_core.djangocms_content as app
# from ..cms_plugins import CMSAllinkContentPlugin, CMSAllinkContentColumnPlugin
#
#
# APP_NAME = app.__name__.split('.')[-1]
# TEMPLATE_DIR = os.path.join(os.path.dirname(app.__file__), 'templates', APP_NAME)
#
# class CMSAllinkContentPluginTest(TestCase):
#
#     def test_plugin_html(self):
#         placeholder = Placeholder.objects.create(slot='test')
#         model_instance = add_plugin(
#             placeholder,
#             CMSAllinkContentPlugin,
#             'en',
#         )
#         renderer = ContentRenderer(request=RequestFactory())
#         html = renderer.render_plugin(model_instance, {})
#         self.assertEqual(html, render_to_string(os.path.join(TEMPLATE_DIR, 'default/content.html')))
#
#
# class CMSAllinkContentColumnPluginTest(TestCase):
#
#     def test_plugin_html(self):
#         placeholder = Placeholder.objects.create(slot='test')
#         model_instance = add_plugin(
#             placeholder,
#             CMSAllinkContentColumnPlugin,
#             'en',
#         )
#         renderer = ContentRenderer(request=RequestFactory())
#         html = renderer.render_plugin(model_instance, {})
#         self.assertEqual(html, render_to_string(os.path.join(TEMPLATE_DIR, 'default/column.html')))
#



# -*- coding: utf-8 -*-
from django import forms
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from webpack_loader.utils import get_files

from .config import MailChimpConfig
from .models import AllinkSignupFormPlugin
from allink_core.core.admin.mixins import AllinkMediaAdminMixin

config = MailChimpConfig()


class AllinkSignupFormPluginForm(forms.ModelForm):

    class Meta:
        model = AllinkSignupFormPlugin
        fields = (
            'signup_form',
        )


@plugin_pool.register_plugin
class CMSAllinkSignupFormPlugin(AllinkMediaAdminMixin, CMSPluginBase):
    model = AllinkSignupFormPlugin
    name = 'Signup Form'
    module = 'allink forms'
    render_template = "allink_mailchimp/plugins/signup_form.html"
    allow_children = False

    form_class = AllinkSignupFormPluginForm
    success_url = '/success/'

    def get_fieldsets(self, request, obj=None):
        fieldsets = super(CMSAllinkSignupFormPlugin, self).get_fieldsets(request, obj)
        return fieldsets

    def get_render_template(self, context, instance, placeholder):
        if instance and instance.signup_form == 'advanced':
            template = 'allink_mailchimp/plugins/signup_form_advanced.html'.format(instance.signup_form)
        else:
            template = self.render_template
        return template

    def render(self, context, instance, placeholder):
        context = super(CMSAllinkSignupFormPlugin, self).render(context, instance, placeholder)
        return context



# -*- coding: utf-8 -*-
import mandrill
from django import forms
from django.conf import settings
from django.utils import translation
from django.utils.translation import ugettext as _
from django.urls import reverse
from django.utils.dateformat import DateFormat
from raven import Client

from allink_core.core.utils import base_url
from allink_core.core.utils import get_display
from allink_core.core_apps.allink_mandrill.config import MandrillConfig
from allink_core.core_apps.allink_mandrill.helpers import check_result_status


class AllinkMandrillEmailBase:
    """
    Sends emails with mandrill.
    The only thing you have to specify is "template_name" the rest of the attributes are optional.

    have a look at the ./README.md for a basic guide on how to implement this class.
    """
    # The template can be sent with two types of merge var language settings
    MERGE_LANG_MAILCHIMP = 'mailchimp'  # Notation: *|FIRST_NAME|*
    MAILCHIMP_FORMAT = "*|{}|*"
    MERGE_LANG_HANDELBAR = 'handlebars'  # Notation: {FIRST_NAME}
    HANDELBAR_FORMAT = "{{{}}}"
    MERGE_FORMAT = {
        MERGE_LANG_MAILCHIMP: MAILCHIMP_FORMAT,
        MERGE_LANG_HANDELBAR: HANDELBAR_FORMAT,

    }
    template_name = None
    # Optional attributes
    merge_language = MERGE_LANG_MAILCHIMP
    translated = True
    google_analytics_campaign = None
    view_content_link = True
    # to make this as flexible as possible we added all the settings options of the Mandrill api message to the attrs
    # I have never seen these changed but if necessary you can do this here
    return_path_domain = None
    preserve_recipients = True
    inline_css = True
    auto_html = None
    auto_text = None

    def __init__(self, template_name=None, logging=False, language=None, async=False, send_with_celery=False,
                 track_clicks=True, track_opens=True, *args, **kwargs):
        """
        Args:
            template_name: mandrill template name (if translated=True, do not add the "_de")
            logging: Boolean if logging is turned on or nor if true implement
                def log(self, kwargs**)
        """
        self.config = MandrillConfig()
        self.logging = logging
        if self.translated:
            assert(language, 'You have to provide a language, if you send a translated email.')
            self.language = language
        self.async = async
        self.send_with_celery = send_with_celery
        if template_name:
            self.template_name = template_name
        self.track_opens = track_opens
        self.track_clicks = track_clicks

    @staticmethod
    def create_global_merge_var(template_keyword, content):
        """
        Args:
            template_keyword: Template Keyword used by Mandrill
            content: Content for this Template Keword

        Returns: Formatted object that meets requirements  for Mandrill API
                {'name': template_keyword, 'content': str(content)}
        """
        # JavaScript expects True like this "true" so we need this thingy
        if content == "False":
            return {'name': str(template_keyword), 'content': False}
        elif content == "True":
            return {'name': str(template_keyword), 'content': True}

        return {'name': str(template_keyword), 'content': str(content)}

    @staticmethod
    def create_email_to_entry(email_address, name):
        return {
            'email': email_address,
            'name': name,
            'type': 'to'
        }

    @staticmethod
    def replace_linebreaks_with_html(content):
        r = '<br />'
        return content.replace('\r\n', r).replace('\n\r', r).replace('\r', r).replace('\n', r)

    def build_subject(self):
        raise NotImplementedError

    def build_body(self):
        raise NotImplementedError

    def get_from_mail_address(self):
        raise NotImplementedError

    def fetch_to_email_addresses(self):
        raise NotImplementedError

    def log(self, **kwargs):
        # this is only needed if self.logging==True
        raise NotImplementedError

    def get_language(self):
        return self.language

    def get_template_name(self):
        assert self.template_name
        if not self.translated:
            return self.template_name
        if self.language:
            template_name = '{}_{}'.format(self.template_name, self.get_language())
        return template_name

    def get_from_name(self):
        return self.config.get_default_from_name()

    def get_website_for_metadata(self):
        return self.config.get_site_domain()

    def get_reply_to_email_address(self):
        return self.get_from_mail_address()

    def get_googleanalytics_domains(self):
        return [self.config.get_site_domain(), ]

    def send_transactional_email(self, message, template_content):
        if self.send_with_celery:

            from .tasks import send_transactional_mail_celery

            template_name = self.get_template_name()
            send_transactional_mail_celery.apply_async(args=[template_name, message, template_content], queue='slow')

        else:
            try:
                mandrill_client = mandrill.Mandrill(self.config.apikey)
                result = mandrill_client.messages.send_template(template_name=self.get_template_name(),
                                                                template_content=template_content, message=message,
                                                                async=self.async)
                check_result_status(result)
            except:  # as we just push it to sentry we catch all the errors
                if not hasattr(settings, 'RAVEN_CONFIG'):
                    raise
                client = Client(settings.RAVEN_CONFIG.get('dsn'))
                client.captureException()
                # sentry is not configured on localhost
                if not settings.RAVEN_CONFIG.get('dsn'):
                    raise

    def send_mail(self):
        """
        This should never throw an exception. Instead it fails silently and sends exceptions to sentry.
        (So should never have to wrap your .send_mail() call in a try/ except.
        for a better handling of failed emails you should implement an appropriate log
        """
        if self.translated:
            prev_language = translation.get_language()
            translation.activate(self.language)
        try:
            template_content = [{}]
            # we can only send, when we have a email address
            if self.fetch_to_email_addresses() and not self.fetch_to_email_addresses() == '':
                message = {
                    'auto_html': self.auto_html,
                    'auto_text': self.auto_text,
                    'from_email': str(self._dev_mode_safe_from_adress()),
                    'from_name': str(self.get_from_name()),
                    'global_merge_vars': self.build_body(),
                    'google_analytics_campaign': self.google_analytics_campaign,
                    'google_analytics_domains': self.get_googleanalytics_domains(),
                    'headers': {'Reply-To': self.get_reply_to_email_address()},
                    'inline_css': self.inline_css,
                    'merge_language': self.merge_language,
                    'metadata': {'website': self.get_website_for_metadata()},
                    'preserve_recipients': self.preserve_recipients,
                    'return_path_domain': self.return_path_domain,
                    'subject': self.build_subject(),
                    'to': self._dev_mode_safe_to_adresses(),
                    'track_clicks': self.track_clicks,
                    'track_opens': self.track_opens,
                    'view_content_link': self.view_content_link,
                }

                self.send_transactional_email(message=message, template_content=template_content)
                # if logging is enabled the function log will be called. You will have to implement it
                if self.logging:
                    self.log(to=message['to'],
                             subject=message['subject'],
                             form=message['from_email'],
                             body_vars=message['global_merge_vars'],
                             full_message=message)
        finally:
            if self.translated:
                translation.activate(prev_language)

    def _fetch_to_email_adresses_dev_mode(self):
        """
        Do not override this method! (unless you really know that this is what you want.)
        """
        return [self.create_email_to_entry(email, 'allink Test')
                for email in settings.ALLINK_MANDRILL_DEV_MODE_TO_EMAIL_ADDRESSES]

    def _get_from_email_adress_dev_mode(self):
        """
        Do not override this method! (unless you really know that this is what you want.)
        """
        return settings.ALLINK_MANDRILL_DEV_MODE_FROM_EMAIL_ADDRESS

    def _dev_mode_safe_from_adress(self):
        """
        Do not override this method! (unless you really know that this is what you want.)
        """
        if settings.ALLINK_MANDRILL_DEV_MODE:
            return self._get_from_email_adress_dev_mode()
        else:
            return self.get_from_mail_address()

    def _dev_mode_safe_to_adresses(self):
        """
        Do not override this method! (unless you really know that this is what you want.)
        """
        if settings.ALLINK_MANDRILL_DEV_MODE:
            return self._fetch_to_email_adresses_dev_mode()
        else:
            return self.fetch_to_email_addresses()


class AllinkMandrillFormEmail(AllinkMandrillEmailBase):
    """
    Takes an additional parameter "form"
    - adds all form fields to the merge vars
    - adds a merge var "form_fields" (this will list all available form fields.
      and can be used in the mailchimp template as a reference)

    APPEND_OPTIONAL_FIELDS = False
      all empty fields won't be send to mandrill
    """
    APPEND_OPTIONAL_FIELDS = True

    def __init__(self, form, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.form = form

    def build_body(self):
        form_fields_merge_vars = self.get_merge_vars_from_form(self.form)

        body = form_fields_merge_vars
        body.append(self.create_global_merge_var('submitted_form_str',
                                                 self.get_submitted_form_str(
                                                     self.form, form_fields_merge_vars)))
        body.append(self.create_global_merge_var('submitted_form_url', self.get_submitted_form_url(self.form)))
        body.append((self.create_global_merge_var('list_of_available_merge_vars',
                                                  self.get_list_of_available_merge_vars(self.form))))
        return body

    def get_merge_vars_from_form(self, form):
        """
        returns a list with all the form fields as a merge vars dict
            [('name': 'foo', 'content': 'bar'), ..]
        """
        form_fields_merge_vars = []
        for field in form.fields:
            if form.data.get(field):
                if type(form.fields.get(field)) == forms.MultipleChoiceField:
                    selected_choices = []
                    for option in form.fields.get(field).choices:
                        name, title = option
                        if name in form.cleaned_data.get(field):
                            selected_choices.append(translation.gettext(title))
                    form_fields_merge_vars.append(self.create_global_merge_var(field, '<br>'.join(selected_choices)))
                elif type(form.fields.get(field)) == forms.DateField:
                    # date field
                    date = DateFormat(form.cleaned_data.get(field)).format('j. F Y')
                    form_fields_merge_vars.append(self.create_global_merge_var(field, date))
                elif hasattr(form.fields.get(field), 'choices'):
                    # model field choices
                    form_fields_merge_vars.append(self.create_global_merge_var(field, get_display(
                        form.data.get(field), form.fields.get(field).choices)))
                elif hasattr(form.fields.get(field).widget, 'choices'):
                    # form field choices
                    form_fields_merge_vars.append(
                        self.create_global_merge_var(field, get_display(form.data.get(field), form.fields.get(
                            field).widget.choices)))
                elif type(form.fields.get(field)) == forms.BooleanField:
                    # checkbox
                    form_fields_merge_vars.append(
                        self.create_global_merge_var(field, _('Yes') if True else _('No')))
                # TODO form.data.get(field) is empty
                # elif type(form.fields.get(field)) == forms.FileField:
                #     # file field
                #     file_link_str = '{}{}'.format(base_url(), form.cleaned_data.get(field).file.url)
                #     form_fields_merge_vars.append(
                #         self.create_global_merge_var(field, file_link_str))
                else:
                    form_fields_merge_vars.append(self.create_global_merge_var(field, form.data.get(field)))
            else:
                # form field empty (optional)
                if self.APPEND_OPTIONAL_FIELDS:
                    form_fields_merge_vars.append(self.create_global_merge_var(field, '—'))
        return form_fields_merge_vars

    def get_submitted_form_str(self, form, form_fields_merge_vars):
        """
        for convenience we add a merge var 'submitted_form'
        (can be used to display all the form fields at once, including labels and linebreaks)
            [('name': 'submitted_form', 'content': '<<string with all form fields and labels separated with <br>'), ..]
        """
        submitted_form_str = '<br>'.join(
            ['{}:<br>{}<br>'.format(form.fields.get(merge_var.get('name')).label,
                                    merge_var.get('content')) for merge_var in form_fields_merge_vars])
        return submitted_form_str

    def get_submitted_form_url(self, form):
        """
        we add a link to the submitted form (url to django admin)
            [('name': 'submitted_form_url', 'content': '<<absolute url to django admin>>'), ..]
        """
        submitted_form_url = '{}{}'.format(base_url(),
                                           reverse('admin:%s_%s_change' % (form.instance._meta.app_label,
                                                                           form.instance._meta.model_name),
                                                   args=[form.instance.id]))
        return submitted_form_url

    def get_list_of_available_merge_vars(self, form):
        """
        we add a merge var 'list_of_available_merge_vars' (can be used as a reference in mailchimp templates)
            [('name': 'form_fields', 'content': '<<string with all form fields separated with
            <br> and formatted in the merge_language>>'), ..]
        """
        form_fields_combined = '<br>'.join(
            [self.MERGE_FORMAT[self.merge_language].format(field) for field in form.fields]
        )
        return form_fields_combined


# TODO should be implemented with the same mindset as AllinkMandrillFormEmail
# class AllinkMandrillPluginEmail(AllinkMandrillEmailBase):
#     """
#     use this, if the plugin defines the email addresses and body text
#     e.g
#
#     """
#     def __init__(self, plugin, *args, **kwargs):
#         super().__init__(*args, **kwargs)
#         self.plugin = plugin



# -*- coding: utf-8 -*-

import logging
import os
import shutil
import textwrap
from os.path import exists, join

import allink_core


def create_local_app_folder(local_app_path):
    if exists(local_app_path):
        raise ValueError("There is already a '%s' folder! Aborting!" % local_app_path)

    for folder in subfolders(local_app_path):
        if not exists(folder):
            os.mkdir(folder)
            init_path = join(folder, '__init__.py')
            if not exists(init_path):
                create_file(init_path)


def subfolders(path):
    """
    Decompose a path string into a list of subfolders

    Eg Convert 'apps/dashboard/ranges' into
       ['apps', 'apps/dashboard', 'apps/dashboard/ranges']
    """
    folders = []
    while path not in ('/', ''):
        folders.append(path)
        path = os.path.dirname(path)
    folders.reverse()
    return folders


def inherit_app_config(local_app_path, app_package, app_label):
    config_name = app_label.title() + 'Config'
    create_file(
        join(local_app_path, '__init__.py'),
        "default_app_config = '{app_package}.config.{config_name}'\n".format(
            app_package=app_package, config_name=config_name))
    create_file(
        join(local_app_path, 'config.py'),
        "from allink_core.apps.{app_label} import config\n\n\n"
        "class {config_name}(config.{config_name}):\n"
        "    name = '{app_package}'\n".format(
            app_package=app_package,
            app_label=app_label,
            config_name=config_name))


def create_file(filepath, content=''):
    with open(filepath, 'w') as f:
        f.write(content)


initial_snippet = '# -*- coding: utf-8 -*-\n'

help_admin = '''
"""
use core:
from allink_core.apps.{app_package}.admin import *  # noqa

override example:
from allink_core.apps.{app_package}.admin import *  # noqa

admin.site.unregister({model_name})

@admin.register({model_name})
class {model_name}Admin({model_name}Admin):
    pass
"""\n'''

help_apps = '''
"""
use core:
from allink_core.apps.{app_package}.cms_apps import *  # noqa

override example:
from allink_core.core.loading import unregister_cms_apps
from allink_core.apps.{app_package}.cms_apps import *  # noqa

unregister_cms_apps({model_name}Apphook)

class {model_name}Apphook({model_name}Apphook):
    def get_urls(self, page=None, language=None, **kwargs):
        urls = super({model_name}Apphook, self).get_urls(page=None, language=None, **kwargs)
        return urls + ['allink_apps.{app_package}.urls']

apphook_pool.register({model_name}Apphook)
"""\n'''

help_menus = '''
"""
use core:
from allink_core.apps.{app_package}.cms_menus import *  # noqa

override example:
from allink_core.core.loading import unregister_cms_menu
from allink_core.apps.{app_package}.cms_menus import *  # noqa

unregister_cms_menu({model_name}Menu)

class {model_name}Menu({model_name}Menu):
   pass

menu_pool.register_menu(get_class('{app_package}.cms_menus', '{model_name}Menu'))
"""\n'''


help_cms_plugins = '''
"""
use core:
from allink_core.apps.{app_package}.cms_plugins import *  # noqa

override example:
from allink_core.apps.{app_package}.cms_plugins import *  # noqa

plugin_pool.unregister_plugin(CMS{model_name}AppContentPlugin)

@plugin_pool.register_plugin
class CMS{model_name}AppContentPlugin(CMS{model_name}AppContentPlugin):
    pass
"""\n'''

help_toolbars = '''
"""
use core:
from allink_core.apps.{app_package}.cms_toolbars import *  # noqa

override example:
from allink_core.allink_base.utils.loading import get_model
from allink_core.apps.{app_package}.cms_toolbars import *  # noqa

from allink_core.core.cms_toolbars import AllinkBaseModifierMixin
Config = get_model('config', 'Config')

toolbar_pool.unregister({model_name}Toolbar)


class {model_name}Toolbar({model_name}Toolbar):
    pass

if getattr(Config.get_solo(), '{app_package}_toolbar_enabled', True):
    toolbar_pool.register({model_name}Toolbar)
    pass
"""\n'''

help_models = '''
"""
use core:
from allink_core.apps.{app_package}.models import *  # noqa

override example:
you don't have to override every model

from django.db import models
from allink_core.apps.{app_package}.abstract_models import Base{model_name}, Base{model_name}Translation, Base{model_name}AppContentPlugin
from allink_core.core.loading import get_model


class {model_name}(Base{model_name}):
    pass


class {model_name}Translation(Base{model_name}Translation):
    pass


class {model_name}AppContentPlugin(Base{model_name}AppContentPlugin):
    data_model = get_model('{app_package}', '{model_name}')
    pass

from allink_core.apps.{app_package}.models import *  # noqa
"""\n'''

help_managers = '''
"""
use core:
from allink_core.apps.{app_package}.managers import *  # noqa

override example:
# -*- coding: utf-8 -*-
from datetime import datetime
from django.db.models import Q
from allink_core.apps.{app_package}.managers import Allink{model_name}QuerySet


class Allink{model_name}QuerySet(Allink{model_name}QuerySet):
    pass


Allink{model_name}Manager = Allink{model_name}QuerySet.as_manager
"""\n'''

help_sitemaps = '''# -*- coding: utf-8 -*-
"""
use core:
no 'sitemaps.py' file needed

override example:
from allink_core.apps.{app_package}.sitemaps import {model_name}Sitemap as Core{model_name}Sitemap

class {model_name}Sitemap(Core{model_name}Sitemap):
    pass
"""\n'''

snippet_urls = '''# -*- coding: utf-8 -*-
"""
use core:
no 'urls.py' file needed

override example:
!important!
make sure you also add the new urls to the cms_apps.py so djangocms knows about them!
ovverride the apphook, see example in cms_apps.py


from django.urls import path
from allink_apps.{app_package}.views import new_view


urlpatterns = [
    path('new-view/<int:id>/', new_view, name='new-view'),
]
"""\n'''

snippet_views = '''# -*- coding: utf-8 -*-
"""
use core:
no 'views.py' file needed

override example:
from allink_core.apps.{app_package}.views import {model_name}Detail as Core{model_name}Detail

class {model_name}Detail(Base{model_name}Detail):
    ...
    def render_to_response(self, context, **response_kwargs):
        if self.request.GET.get('softpage', None):
            context.update({'base_template': 'app_content/ajax_base.html'})
        context.update({'hello': 'hello view is correctly overridden!!'})
        return render_to_response(self.get_template_names(), context, context_instance=RequestContext(self.request))

# TO ADD NEW VIEW:
import datetime
from django.http import HttpResponse
from allink_core.core.loading import get_model
from allink_core.apps.{app_package}.pdf import Pdf{model_name}

{model_name} = get_model('{app_package}', '{model_name}')

def export_pdf(request, id):
    date = (datetime.date.today().strftime('%d-%m-%Y'))

    item = {model_name}.objects.get(id=id)

    pdf = Pdf{model_name}(item, request)
    output = pdf.build()
    filename = '%s_%s.pdf' % (item.title, date)

    response = HttpResponse(output.getvalue(), content_type="application/pdf")
    response['Content-Disposition'] = "attachment; filename=%s" % filename
    return response
"""\n'''


def fork_app(label, folder_path, logger=None, help=False):  # noqa
    """
    Create a custom version of one of allink's apps
    (help: Including helpful comments to get you started)

    """

    # Create minimum app files
    app_package = label
    model_name = label.title().replace('_', '')

    # Create all snippets
    snippet_admin = initial_snippet
    snippet_cms_apps = initial_snippet
    snippet_menus = initial_snippet
    snippet_cms_plugins = initial_snippet
    snippet_toolbars = initial_snippet
    snippet_managers = initial_snippet
    snippet_models = initial_snippet
    snippet_sitemaps = initial_snippet

    if help:
        snippet_admin += help_admin.format(app_package=label, model_name=model_name)
        snippet_cms_apps += help_apps.format(app_package=label, model_name=model_name)
        snippet_menus += help_menus.format(app_package=label, model_name=model_name)
        snippet_cms_plugins += help_cms_plugins.format(app_package=label, model_name=model_name)
        snippet_toolbars += help_toolbars.format(app_package=label, model_name=model_name)
        snippet_managers += help_managers.format(app_package=label, model_name=model_name)
        snippet_models += help_models.format(app_package=label, model_name=model_name)
        snippet_sitemaps += help_sitemaps.format(app_package=label, model_name=model_name)

    snippet_admin += "from allink_core.apps.{}.admin import *  # noqa\n".format(label)
    snippet_cms_apps += "from allink_core.apps.{}.cms_apps import *  # noqa\n".format(label)
    snippet_menus += "from allink_core.apps.{}.cms_menus import *  # noqa\n".format(label)
    snippet_cms_plugins += "from allink_core.apps.{}.cms_plugins import *  # noqa\n".format(label)
    snippet_toolbars += "from allink_core.apps.{}.cms_toolbars import *  # noqa\n".format(label)
    snippet_managers += "from allink_core.apps.{}.managers import *  # noqa\n".format(label)
    snippet_models += "from allink_core.apps.{}.models import *  # noqa\n".format(label)

    if logger is None:
        logger = logging.getLogger(__name__)

    # Check label is valid
    valid_labels = [x.replace('allink_core.apps.', '')
                    for x in allink_core.ALLINK_CORE_ALLINK_APPS if x.startswith('allink_core')]
    if label not in valid_labels:
        raise ValueError("There is no allink_core app that matches '%s'" % label)

    # Check folder_path is current catalog
    if folder_path == '.':
        folder_path = ''

    # Create folder
    label_folder = label.replace('.', '/')
    local_app_path = join(folder_path, label_folder)
    logger.info("Creating package %s" % local_app_path)
    create_local_app_folder(local_app_path)

    # Create minimum app files
    app_package = local_app_path.replace('/', '.')

    allink_core_app_path = join(allink_core.__path__[0], 'apps', label_folder)
    if exists(os.path.join(allink_core_app_path, 'admin.py')):
        logger.info("Creating admin.py")
        create_file(join(local_app_path, 'admin.py'), snippet_admin)

    logger.info("Creating app config")
    inherit_app_config(local_app_path, app_package, label)

    # Only create models.py and migrations if it exists in the allink_core app
    allink_core_models_path = join(allink_core_app_path, 'models.py')
    if exists(allink_core_models_path):
        logger.info("Creating models.py")
        create_file(
            join(local_app_path, 'models.py'), snippet_models)

        migrations_path = 'migrations'
        source = join(allink_core_app_path, migrations_path)
        if exists(source):
            logger.info("Creating %s folder", migrations_path)
            destination = join(local_app_path, migrations_path)
            shutil.copytree(source, destination)

    # cms_apps.py
    if exists(os.path.join(allink_core_app_path, 'cms_apps.py')):
        logger.info("Creating cms_apps.py")
        create_file(join(local_app_path, 'cms_apps.py'), snippet_cms_apps)

    # cms_menus.py
    if exists(os.path.join(allink_core_app_path, 'cms_menus.py')):
        logger.info("Creating cms_menus.py")
        create_file(join(local_app_path, 'cms_menus.py'), snippet_menus)

    # cms_plugins.py
    if exists(os.path.join(allink_core_app_path, 'cms_plugins.py')):
        logger.info("Creating admin.py")
        create_file(join(local_app_path, 'cms_plugins.py'), snippet_cms_plugins)

    # cms_toolbars.py
    if exists(os.path.join(allink_core_app_path, 'cms_toolbars.py')):
        logger.info("Creating cms_toolbars.py")
        create_file(join(local_app_path, 'cms_toolbars.py'), snippet_toolbars)

    # only for help
    if help:
        if exists(os.path.join(allink_core_app_path, 'sitemaps.py')):
            logger.info("Creating sitemaps.py")
            create_file(join(local_app_path, 'sitemaps.py'), snippet_sitemaps)

        if exists(os.path.join(allink_core_app_path, 'managers.py')):
            logger.info("Creating managers.py")
            create_file(join(local_app_path, 'managers.py'), snippet_managers)

        if exists(os.path.join(allink_core_app_path, 'urls.py')):
            logger.info("Creating urls.py")
            create_file(join(local_app_path, 'urls.py'), snippet_urls)

        if exists(os.path.join(allink_core_app_path, 'views.py')):
            logger.info("Creating views.py")
            create_file(join(local_app_path, 'views.py'), snippet_views)

    # Final step needs to be done by hand
    msg = (
        "The final step is to uncomment '%s' in OVERRIDDEN_ALLINK_CORE_APPS "
        "(replacing the equivalent allink_core app). e.g.:"
    ) % app_package
    snippet = (
        "  # settings.py\n"
        "  ...\n"
        "  OVERRIDDEN_ALLINK_CORE_APPS = [\n"
        "      ['%s'],"
        "      ...\n"
        "  ]\n"
    ) % app_package
    record = "\n%s\n\n%s" % (
        "\n".join(textwrap.wrap(msg)), snippet)
    logger.info(record)


new_admin = '''# -*- coding: utf-8 -*-
from django.contrib import admin
from django import forms
from adminsortable.admin import SortableAdmin
from cms.admin.placeholderadmin import PlaceholderAdminMixin
from parler.admin import TranslatableAdmin
from allink_core.core.admin import AllinkMediaAdminMixin, AllinkSEOAdminMixin, AllinkCategoryAdminMixin, AllinkTeaserAdminMixin, AllinkCategoryAdminForm

from {app_package}.models import {model_name}


@admin.register({model_name})
class {model_name}Admin( AllinkMediaAdminMixin, AllinkSEOAdminMixin, AllinkCategoryAdminMixin, AllinkTeaserAdminMixin, PlaceholderAdminMixin, TranslatableAdmin, SortableAdmin):
    list_filter = ('status', 'categories',)

    def get_fieldsets(self, request, obj=None):
        fieldsets = (
            (None, {{
                'fields': (
                    'status',
                    'title',
                    'slug',
                    'lead',
                    'preview_image',
                )
            }}),
        )

        fieldsets += self.get_category_fieldsets()
        fieldsets += self.get_teaser_fieldsets()
        fieldsets += self.get_seo_fieldsets()
        return fieldsets

    def formfield_for_dbfield(self, db_field, **kwargs):
        if db_field.name == 'lead':
            kwargs['widget'] = forms.Textarea
        return super({model_name}Admin, self).formfield_for_dbfield(db_field, **kwargs)

'''

new_cms_apps = '''# -*- coding: utf-8 -*-
from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool
from django.utils.translation import ugettext_lazy as _


class {model_name}Apphook(CMSApp):
    name = _("{model_name} Apphook")
    app_name = '{label}'

    def get_urls(self, page=None, language=None, **kwargs):
        return ['{app_package}.urls']


apphook_pool.register({model_name}Apphook)

'''

new_cms_menu = '''# -*- coding: utf-8 -*-
from menus.base import NavigationNode
from menus.menu_pool import menu_pool
from django.utils.translation import ugettext_lazy as _
from cms.menu_bases import CMSAttachMenu

from {app_package}.models import {model_name}


class {model_name}Menu(CMSAttachMenu):

    name = _("{model_name} menu")

    def get_nodes(self, request):

        nodes = []
        for entry in {model_name}.objects.active():
            node = NavigationNode(
                entry.title,
                entry.get_absolute_url(),
                entry.sort_order,
                attr={{
                    'description': entry.lead
                }})
            nodes.append(node)

        return nodes


menu_pool.register_menu('{app_package}.cms_menus', '{model_name}Menu')

'''

new_cms_plugins = '''# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from cms.plugin_pool import plugin_pool
from cms.plugin_base import CMSPluginBase
from allink_core.core.cms_plugins import CMSAllinkBaseAppContentPlugin

from {app_package}.models import {model_name}AppContentPlugin


@plugin_pool.register_plugin
class CMS{model_name}AppContentPlugin(CMSAllinkBaseAppContentPlugin):
    """
    model:
    - where to store plugin instances

    name:
    - name of the plugin
    """
    model = {model_name}AppContentPlugin
    name = model.data_model._meta.verbose_name_plural

'''

new_cms_toolbars = '''# -*- coding: utf-8 -*-
from cms.toolbar_pool import toolbar_pool
from cms.toolbar_base import CMSToolbar

from allink_core.core.cms_toolbars import AllinkBaseModifierMixin

from {app_package}.models import {model_name}


class {model_name}Toolbar(AllinkBaseModifierMixin, CMSToolbar):
    model = {model_name}
    app_label = {model_name}._meta.app_label


toolbar_pool.register({model_name}Toolbar)

'''

new_config = '''# -*- coding: utf-8 -*-
from django.apps import AppConfig


class {model_name}Config(AppConfig):
    name = '{app_package}'
    verbose_name = "{model_name}"

'''

new_forms = '''# -*- coding: utf-8 -*-
'''

new_models = '''# -*- coding: utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from django.db import models

from cms.models.fields import PageField
from cms.models.pluginmodel import CMSPlugin
from cms.models.fields import PlaceholderField
from adminsortable.models import SortableMixin
from parler.models import TranslatedField
from djangocms_text_ckeditor.fields import HTMLField
from filer.fields.image import FilerImageField

from aldryn_common.admin_fields.sortedm2m import SortedM2MModelField
from allink_core.core.models import (
    AllinkCategoryFieldsModel,
    AllinkBaseTranslatableModel,
    AllinkBaseAppContentPlugin,
    AllinkBaseTranslatedFieldsModel,
)

from allink_core.core.models.managers import AllinkBaseModelQuerySet


class {model_name}(SortableMixin, AllinkCategoryFieldsModel, AllinkBaseTranslatableModel):
    slug_source_field_name = 'title'

    title = TranslatedField(any_language=True)
    lead = TranslatedField()

    preview_image = FilerImageField(
        verbose_name='Preview Image',
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        related_name='%(app_label)s_%(class)s_preview_image',
    )
    sort_order = models.PositiveIntegerField(
        default=0,
        editable=False,
        db_index=True
    )

    header_placeholder = PlaceholderField(
       '{label}_header',
        related_name='%(app_label)s_%(class)s_header_placeholder'
    )
    content_placeholder = PlaceholderField(
       '{label}_content',
        related_name='%(app_label)s_%(class)s_content_placeholder'
    )

    objects = AllinkCategoryModelManager()

    class Meta:
        app_label = '{label}'
        ordering = ('sort_order',)
        verbose_name = '{model_name}'
        verbose_name_plural = '{model_name}'


class {model_name}Translation(AllinkBaseTranslatedFieldsModel):
    master = models.ForeignKey(
        '{label}.{model_name}',
        on_delete=models.CASCADE,
        related_name='translations',
        null=True
        )

    title = models.CharField(
        max_length=255
    )
    lead = HTMLField(
        _('Lead Text'),
        help_text='Teaser text that in some cases is used in the list view and/or in the detail view.',
        blank=True,
        null=True,
    )
    class Meta:
        app_label = '{label}'


class {model_name}AppContentPlugin(AllinkBaseAppContentPlugin):
    data_model = {model_name}

    manual_entries = SortedM2MModelField(
        '{label}.{model_name}',
        blank=True,
        help_text=('Select and arrange specific entries, or, leave blank to select all. (If '
                    'manual entries are selected the category filtering will be applied as well.)'
    )
    apphook_page = PageField(
        verbose_name='Apphook Page',
        null=True,
        blank=True,
        on_delete=models.PROTECT,
        help_text='If provided, this Apphook-Page will be used to generate the detail link.',
    )

    class Meta:
        app_label = '{label}'

'''

new_managers = '''# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
from allink_core.core.models.managers import AllinkCategoryModelQuerySet


class Allink{model_name}QuerySet(AllinkCategoryModelQuerySet):
    pass


Allink{model_name}Manager = Allink{model_name}QuerySet.as_manager

'''

new_sitemaps = '''# -*- coding: utf-8 -*-
from django.contrib.sitemaps import Sitemap

from {app_package}.models import {model_name}


class {model_name}Sitemap(Sitemap):

    changefreq = "never"
    priority = 0.5
    i18n = True

    def __init__(self, *args, **kwargs):
        self.namespace = kwargs.pop('namespace', None)
        super({model_name}Sitemap, self).__init__(*args, **kwargs)

    def items(self):
        return {model_name}.objects.active()

    def lastmod(self, obj):
        return obj.modified

'''

new_urls = '''# # -*- coding: utf-8 -*-
from django.urls import path

from {app_package}.views import {model_name}PluginLoadMore, {model_name}Detail


urlpatterns = [
    path('<int:page>/', {model_name}PluginLoadMore.as_view(), name='more'),
    path('<slug:slug>/', {model_name}Detail.as_view(), name='detail'),
]

'''

new_views = '''# -*- coding: utf-8 -*-
from allink_core.core.views import AllinkBasePluginLoadMoreView, AllinkBaseDetailView, AllinkBaseAjaxFormView

from {app_package}.models import {model_name}, {model_name}AppContentPlugin


class {model_name}PluginLoadMore(AllinkBasePluginLoadMoreView):
    model = {model_name}
    plugin_model = {model_name}AppContentPlugin


class {model_name}Detail(AllinkBaseDetailView):
    model = {model_name}

'''

def new_app(label, folder_path, logger=None):
    """
    Create a new app with all the basic functionality a allink app has.
    """

    if logger is None:
        logger = logging.getLogger(__name__)

    # Check folder_path is current catalog
    if folder_path == '.':
        folder_path = ''

    # Create folder
    label_folder = label.replace('.', '/')
    local_app_path = join(folder_path, label_folder)
    logger.info("Creating package %s" % local_app_path)
    create_local_app_folder(local_app_path)

    # Create minimum app files
    app_package = local_app_path.replace('/', '.')
    model_name = label.title().replace('_', '')

    # Create all snippets
    snippet_admin = new_admin.format(label=label, app_package=app_package, model_name=model_name)
    snippet_cms_apps = new_cms_apps.format(label=label, app_package=app_package, model_name=model_name)
    snippet_cms_menu = new_cms_menu.format(label=label, app_package=app_package, model_name=model_name)
    snippet_cms_plugins = new_cms_plugins.format(label=label, app_package=app_package, model_name=model_name)
    snippet_cms_toolbars = new_cms_toolbars.format(label=label, app_package=app_package, model_name=model_name)
    snippet_config = new_config.format(label=label, app_package=app_package, model_name=model_name)
    snippet_forms = new_forms.format(label=label, app_package=app_package, model_name=model_name)
    snippet_models = new_models.format(label=label, app_package=app_package, model_name=model_name)
    snippet_managers = new_managers.format(label=label, app_package=app_package, model_name=model_name)
    snippet_sitemaps = new_sitemaps.format(label=label, app_package=app_package, model_name=model_name)
    snippet_urls = new_urls.format(label=label, app_package=app_package, model_name=model_name)
    snippet_views = new_views.format(label=label, app_package=app_package, model_name=model_name)

    logger.info("Creating admin.py")
    create_file(join(local_app_path, 'admin.py'), snippet_admin)

    logger.info("Creating cms_apps.py")
    create_file(join(local_app_path, 'cms_apps.py'), snippet_cms_apps)

    logger.info("Creating cms_menu.py")
    create_file(join(local_app_path, 'cms_menu.py'), snippet_cms_menu)

    logger.info("Creating cms_plugins.py")
    create_file(join(local_app_path, 'cms_plugins.py'), snippet_cms_plugins)

    logger.info("Creating cms_toolbars.py")
    create_file(join(local_app_path, 'cms_toolbars.py'), snippet_cms_toolbars)

    logger.info("Creating config.py")
    create_file(join(local_app_path, 'config.py'), snippet_config)

    logger.info("Creating forms.py")
    create_file(join(local_app_path, 'forms.py'), snippet_forms)

    logger.info("Creating models.py")
    create_file(join(local_app_path, 'models.py'), snippet_models)

    logger.info("Creating managers.py")
    create_file(join(local_app_path, 'managers.py'), snippet_managers)

    logger.info("Creating sitemaps.py")
    create_file(join(local_app_path, 'sitemaps.py'), snippet_sitemaps)

    logger.info("Creating urls.py")
    create_file(join(local_app_path, 'urls.py'), snippet_urls)

    logger.info("Creating views.py")
    create_file(join(local_app_path, 'views.py'), snippet_views)

    # Final step needs to be done by hand
    msg = (
        """
            The final steps:\n
            1. add '{app_package}' to PROJECT_APPS \n
            2. add Plugins to ALLINK_CONTENT_PLUGIN_CHILD_CLASSES \n
            3. define templates/ create a new tuple {model_name_uppper}_PLUGIN_TEMPLATES \n
            4. (optional) add '('{model_name_lower}', '{model_name}'),'
            to PROJECT_APP_MODEL_WITH_CATEGORY_CHOICES if the app should have categories \n
            5. create all required templates (default grid_static, detail and no_result)
            6. ./manage.py makemigrations {label} ./manage.py migrate\n\n
        """
    ).format(
        model_name=model_name,
        model_name_uppper=str.upper(model_name),
        model_name_lower=str.lower(model_name),
        app_package=app_package,
        label=label
    )

    logger.info(msg)



# -*- coding: utf-8 -*-

from django.db import models
from django.contrib.auth.models import Group
from django.utils.translation import ugettext_lazy as _
from cms.models.pluginmodel import CMSPlugin


class AllinkGroupContainerPlugin(CMSPlugin):
    """
    A Container-Plugin for Group of Content
    """

    title = models.CharField(
        'Title',
        max_length=255
    )

    groups = models.ManyToManyField(
        Group,
        blank=True,
    )

    def __str__(self):
        return self.title

    def copy_relations(self, oldinstance):
        for i in oldinstance.groups.all():
            self.groups.add(i)


class AllinkGroupPlugin(CMSPlugin):
    title = models.CharField(
        'Title',
        max_length=255
    )

    def __str__(self):
        return '{}'.format(self.title)



# -*- coding: utf-8 -*-
from django.contrib.sitemaps import Sitemap

from allink_core.core.loading import get_model


Locations = get_model('locations', 'Locations')


class LocationsSitemap(Sitemap):

    changefreq = "never"
    priority = 0.5
    i18n = True

    def __init__(self, *args, **kwargs):
        self.namespace = kwargs.pop('namespace', None)
        super(LocationsSitemap, self).__init__(*args, **kwargs)

    def items(self):
        return Locations.objects.active()

    def lastmod(self, obj):
        return obj.modified



# -*- coding: utf-8 -*-
from django.urls import reverse
from django.contrib.auth import get_permission_codename

from cms.toolbar_pool import toolbar_pool
from cms.toolbar_base import CMSToolbar

from allink_core.core.cms_toolbars import AllinkBaseModifierMixin
from allink_core.core.loading import get_model

Events = get_model('events', 'Events')
EventsRegistration = get_model('events', 'EventsRegistration')


class EventsToolbar(AllinkBaseModifierMixin, CMSToolbar):
    model = Events
    app_label = Events._meta.app_label


toolbar_pool.register(EventsToolbar)


class EventsRegistrationToolbar(CMSToolbar):
    model = EventsRegistration

    def populate(self):
        opts = self.model._meta

        if self.request.user.has_perm('%s.%s' % (opts.app_label, get_permission_codename('change', opts))):
            menu = self.toolbar.get_or_create_menu('form-menu', 'Forms')
            url = reverse('admin:{}_{}_changelist'.format(self.model._meta.app_label, self.model._meta.model_name))
            menu.add_sideframe_item(self.model._meta.verbose_name_plural, url=url)


toolbar_pool.register(EventsRegistrationToolbar)



# -*- coding: utf-8 -*-
from django import forms
from django.contrib.postgres.forms import SplitArrayField
from cms.plugin_base import CMSPluginBase

from cms.plugin_pool import plugin_pool
from webpack_loader.utils import get_files

from allink_core.core.utils import get_additional_choices
from allink_core.core.loading import get_model
from allink_core.core.admin.mixins import AllinkMediaAdminMixin

ContactRequestPlugin = get_model('contact', 'ContactRequestPlugin')


class ContactRequestFormPluginForm(AllinkMediaAdminMixin, forms.ModelForm):
    internal_email_addresses = SplitArrayField(forms.EmailField(required=False), size=3)

    class Meta:
        model = ContactRequestPlugin
        fields = ('send_internal_mail', 'internal_email_addresses', 'from_email_address', 'send_external_mail',
                  'thank_you_text', 'label_layout', 'project_css_classes')

    def __init__(self, *args, **kwargs):
        super(ContactRequestFormPluginForm, self).__init__(*args, **kwargs)
        if get_additional_choices('FORM_CSS_CLASSES'):
            self.fields['project_css_classes'] = forms.MultipleChoiceField(
                widget=forms.CheckboxSelectMultiple(),
                label='Predefined variations',
                choices=get_additional_choices('FORM_CSS_CLASSES'),
                required=False,
            )
        else:
            self.fields['project_css_classes'].widget = forms.widgets.HiddenInput()


@plugin_pool.register_plugin
class CMSAllinkContactRequestPlugin(CMSPluginBase):
    model = ContactRequestPlugin
    name = 'Contact Form'
    module = 'allink forms'
    form = ContactRequestFormPluginForm

    def get_render_template(self, context, instance, placeholder):
        template = 'contact/content.html'
        return template



# -*- coding: utf-8 -*-
import factory
from allink_core.core.test.factories import FilerImageFactory
from ..models import Config, AllinkPageExtension, AllinkTitleExtension


class ConfigFactory(factory.DjangoModelFactory):

    class Meta:
        model = Config

    default_og_image = factory.SubFactory(FilerImageFactory)
    # theme_color
    # mask_icon_color
    # msapplication_tilecolor
    google_site_verification = 'google-site-verification-code'
    # default_to_email
    # default_from_email
    default_base_title = 'default base title'


class AllinkPageExtensionFactory(factory.DjangoModelFactory):

    class Meta:
        model = AllinkPageExtension

    extended_object = None
    og_image = factory.SubFactory(FilerImageFactory)
    teaser_image = factory.SubFactory(FilerImageFactory)


class AllinkTitleExtensionFactory(factory.DjangoModelFactory):

    class Meta:
        model = AllinkTitleExtension

    extended_object = None
    og_title = 'og title'
    og_description = 'og description'
    teaser_title = 'teaser title'
    teaser_technical_title = 'teaser technical title'
    teaser_description = 'teaser description'
    teaser_link_text = 'teaser link text'




# -*- coding: utf-8 -*-
from django.contrib import admin
from django.contrib.admin.utils import unquote
from django.shortcuts import redirect
from django.urls import path

from parler.admin import TranslatableAdmin

from allink_core.core_apps.allink_terms.models import AllinkTerms


@admin.register(AllinkTerms)
class AllinkTermsAdmin(TranslatableAdmin, admin.ModelAdmin):
    list_display = ('__str__', 'status', 'start', 'end')
    fields = ('text', 'terms_cms_page')
    actions = None
    change_form_template = 'allink_terms/admin/change_form.html'

    def has_delete_permission(self, request, obj=None):
        if obj and not obj.is_publishable:
            return False
        return super(AllinkTermsAdmin, self).has_delete_permission(request, obj)

    def add_view(self, request, form_url='', extra_context=None):
        self.readonly_fields = []
        return super(AllinkTermsAdmin, self).add_view(request, form_url, extra_context)

    def change_view(self, request, object_id, form_url='', extra_context=None):
        instance = self.get_object(request, unquote(object_id))
        extra_context = extra_context or {}
        extra_context['show_save'] = False
        extra_context['show_save_as_new'] = False
        extra_context['hide_save_and_add_another'] = True
        extra_context['hide_save_and_continue'] = True
        if instance and instance.status != AllinkTerms.STATUS_DRAFT:
            self.readonly_fields = ['text', 'terms_cms_page']
        else:
            self.readonly_fields = []
        return super(AllinkTermsAdmin, self).change_view(request, object_id, form_url, extra_context)

    def publish(self, request, object_id):
        instance = self.get_object(request, object_id)
        if instance:
            instance.publish()
        info = self.model._meta.app_label, self.model._meta.model_name
        return redirect('admin:%s_%s_changelist' % info)

    def get_urls(self):
        urls = super(AllinkTermsAdmin, self).get_urls()
        my_urls = [
            path('publish/', self.admin_site.admin_view(self.publish))
        ]
        return my_urls + urls



# -*- coding: utf-8 -*-
from django import forms

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from allink_core.core_apps.allink_terms.models import AllinkTerms
from allink_core.core_apps.allink_terms.models import AllinkTermsPlugin


class AllinkTermsPluginForm(forms.ModelForm):

    class Meta:
        model = AllinkTermsPlugin
        exclude = ('page', 'position', 'placeholder', 'language', 'plugin_type')


@plugin_pool.register_plugin
class CMSAllinkTermsPlugin(CMSPluginBase):
    model = AllinkTermsPlugin
    name = 'Terms of Service'
    module = 'allink modules'
    form = AllinkTermsPluginForm
    render_template = 'allink_terms/plugins/terms.html'

    def render(self, context, instance, placeholder):

        context['instance'] = instance
        context['placeholder'] = placeholder
        context['current_terms'] = AllinkTerms.objects.get_published()

        return context



# -*- coding: utf-8 -*-
from allink_core.apps.events.abstract_models import (
    BaseEvents,
    BaseEventsTranslation,
    BaseEventsAppContentPlugin,
    BaseEventsRegistration
)
from allink_core.core.loading import get_model
from allink_core.core.loading import is_model_registered

__all__ = []


# Events
if not is_model_registered('events', 'Events'):
    class Events(BaseEvents):
        pass

    __all__.append('Events')


if not is_model_registered('events', 'EventsTranslation'):
    class EventsTranslation(BaseEventsTranslation):
        pass

    __all__.append('EventsTranslation')


if not is_model_registered('events', 'EventsAppContentPlugin'):
    class EventsAppContentPlugin(BaseEventsAppContentPlugin):
        data_model = get_model('events', 'Events')

    __all__.append('EventsAppContentPlugin')


if not is_model_registered('events', 'EventsRegistration'):
    class EventsRegistration(BaseEventsRegistration):
        pass

    __all__.append('EventsRegistration')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.contrib.admin.widgets import AdminFileWidget
from django.utils.safestring import mark_safe
from django.conf import settings
from django import forms
from webpack_loader.utils import get_files

from allink_core.core.utils import get_project_color_choices
from allink_core.core.admin.mixins import AllinkMediaAdminMixin


class AdminPdfThumnailWidget(AdminFileWidget):
    def render(self, name, value, attrs=None):
        output = []

        if value:
            output.append('<img alt="%s" src="%s" height="200"/>' % (value.url, value.url,))
        else:
            output.append('Thumbnail is created when publication is saved. And a PDF document is assigned.')

        # This is commented out b/c maybe you want to be able to override the thumbnail?
        # output.append(super(AdminFileWidget, self).render(name, value, attrs))

        return mark_safe(''.join(output))


class Icon(forms.widgets.TextInput):
    def render(self, name, value, attrs=None, **kwargs):
        input_html = super(Icon, self).render(name, value, attrs=attrs, **kwargs)
        if value is None:
            value = ''
        iconset = value.split('-')[0] if value and '-' in value else ''
        iconset_prefexes = [s[1] for s in settings.PROJECT_ICONSETS]
        if len(settings.PROJECT_ICONSETS) and iconset not in iconset_prefexes:
            # invalid iconset! maybe because the iconset was removed from
            # the project. set it to the first in the list.
            iconset = settings.PROJECT_ICONSETS[0][1]
        from django.template.loader import render_to_string
        rendered = render_to_string(
            'admin/allink_button_link/widgets/icon.html',
            {
                'input_html': input_html,
                'value': value,
                'name': name,
                'iconset': iconset,
                'is_required': self.is_required,
                'iconsets': settings.PROJECT_ICONSETS,
            },
        )
        return rendered


class SpectrumColorPicker(AllinkMediaAdminMixin, forms.widgets.TextInput):
    """
    Based on Brian Grinstead's Spectrum - http://bgrins.github.com/spectrum/
    This widget is used to select a Project Color. With some few options
    in the pushed colorFields, it could be used more flexible if needed.
    """

    def __init__(self, *args, **kwargs):
        self.default = kwargs.pop('default', None)
        super(SpectrumColorPicker, self).__init__(*args, **kwargs)

    def _get_project_color_choices(self):
        palette = ",".join("'%s'" % color for color in get_project_color_choices())
        if self.default:
            palette = "'%s'," % self.default + palette
        else:
            palette = palette + ",'transparent'"
        return palette

    def _render_js(self, _id, value):
        js = """
            <script type="text/javascript">
                document.addEventListener("DOMContentLoaded", function(event) {
                    if (!window.colorFields) {
                        window.colorFields = [];
                    }

                    window.colorFields.push({
                        id: '#%s',
                        color: "%s",
                        showPaletteOnly: true,
                        palette:[%s],
                        localStorageKey: "projectcolors"
                    });
                });
            </script>""" % (_id, value, self._get_project_color_choices())
        return js

    def render(self, name, value, attrs=None, renderer=None):
        if 'id' not in attrs:
            attrs['id'] = "id_%s" % name
        if value:
            for key, val in get_project_color_choices().items():
                if val == value:
                    value = key
                    break
        rendered = super(SpectrumColorPicker, self).render(name, value, attrs=attrs, renderer=None)
        return mark_safe(rendered + self._render_js(attrs['id'], value))


class SearchSelectWidget(AllinkMediaAdminMixin, forms.widgets.Select):

    def __init__(self, attrs=None, choices=()):
        class_attrs = {"data-live-search": "true", 'class': 'selectpicker'}
        if attrs:
            attrs.update(class_attrs)
        else:
            attrs = class_attrs
        super(SearchSelectWidget, self).__init__(attrs)
